from django.urls import path

from .views import Checker, Charge, ViewTransaction, Verification

urlpatterns = [
    path('', Checker.as_view()),
    path('charge', Charge.as_view()),
    path('transaction-history/<str:customerName>', ViewTransaction.as_view()),
    path('verify',Verification.as_view()),
]



from rest_framework import serializers


class ChargeBodySerializer(serializers.Serializer):
    """
    This class serializes the body data that is to be sent in the request 
    for charge end point
    """
    amount = serializers.CharField(max_length="255")
    currency = serializers.CharField(max_length="255")
    phone_number = serializers.CharField(max_length="255")
    email = serializers.CharField(max_length="255")
    tx_ref = serializers.CharField(max_length="255")
    fullName = serializers.CharField(max_length="255")
    network = serializers.CharField(max_length="255")
    redirect_url = serializers.CharField(max_length="255")
    description = serializers.CharField(max_length="255")



from urllib.parse import parse_qs, urlparse
from django.shortcuts import render
import requests
import json
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.exceptions import AuthenticationFailed
from rest_framework import status
from .utils import requestNeeds
from .serializers import ChargeBodySerializer
from .types import ChargeBody
from .errors import RequestError


class Checker(APIView):
    """
    Checker class makes a get request which returns a welcome test to show you
    that you have succefuly installed the app to your project.
    """

    def get(self, requests):
        return Response("When you see this message that means that you have installed me successfuly")


class Charge(APIView):
    """
    Charge is a class that sends a post request to flutterwave
    API to charge money from users using mobile money uganda
    """

    def post(self, request):
        requestInfo = requestNeeds(
            "https://api.flutterwave.com/v3/charges?type=mobile_money_uganda")

        data = ChargeBody(request.data['amount'],
                          request.data['currency'],
                          request.data['phoneNumber'],
                          request.data['email'],
                          request.data['fullName'],
                          request.data['network'],
                          request.data['redirect_url'],# the redirect_url is the webhook url
                          request.data['description']
                          )
        serializer = ChargeBodySerializer(data.chargeBodyObject())
        jsonBody = json.dumps(serializer.data)

        sender = requests.post(
            url=f"{requestInfo['url']}", data=jsonBody, headers=dict(requestInfo['headers']))

        data = sender.json()
        if data['status'] != "success":
            return Response(data, status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response(data)


class ViewTransaction(APIView):
    """
    This class is responsible for view sepcific 
    """

    def get(self, request, customerName: str):
        requestInfo = requestNeeds(
            "https://api.flutterwave.com/v3/transactions", customerName)

        responseData = requests.get(
            url=requestInfo['url'], params=dict(requestInfo['params']), headers=dict(requestInfo['headers']))

        try:
            if not responseData.ok:
                raise RequestError
        except RequestError:
            return Response({"error": "error happened from flutterwave"})

        response = Response()
        response.data = responseData.json()
        return response


class Verification(APIView):
    """
    Verification class acts as a webhook which recieves response of a transaction made.
    It verifies if the transaction is done successfuly or not.
    It returns a dictionary of data and verification status where data is a dictionary 
    of data returned and verification status is a bolean 
    """

    def get(self, request):
        verificationStatus = False
        url = request.build_absolute_uri()
        parsed_url = urlparse(url)
        captured_value = parse_qs(parsed_url.query)['resp'][0]
        data = json.loads(captured_value)
        if data["status"] == "success":
            if data["data"]["status"] == "successful":
                verificationStatus = True
            return Response({"verificationStatus": verificationStatus, "data": data}, status=status.HTTP_200_OK)
        else:
            return Response({"error": "Something went wrong"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



import uuid
from django.conf import settings


def uuidGenerator():
    """
    this function is for generating and returning a
    uuid
    """
    txRef = str(uuid.uuid1())
    return txRef


def getToken():
    """
    getToken is the functtion that returns 
    public_key token which is used for authorization when
    making requests
    """
    token = str(settings.PUBLIC_KEY)
    return token


def requestNeeds(url: str, customerName: str = "") -> dict:
    """
    requestNeeds returns a dictionary of url and headers.
    token value is the API key that set in the settings files of the project.
    searchValue is only passed when you are calling a transaction history for a single 
    person
    """
    token = getToken()
    setUrl = url
    headers = {'Content-type': 'application/json',
               'Authorization': f"Bearer {token}"}
    params = {
        "customer_fullname": customerName
    }

    return {
        'url': setUrl,
        'headers': headers,
        'params': params
    }



from .utils import uuidGenerator

class ChargeBody:
    """
    ChargeBody class takes in arguments of data and creates 
    an object out the passed data. the object created is for 
    the charge end point to be sent.
    """

    def __init__(self, amount: str, currency: str, phone_number: str,
                 email: str, fullName: str, network: str, redirect_url: str,
                 description: str) -> None:
        # Asign to self object
        self.amount = amount
        self.currency = currency
        self.phone_number = phone_number
        self.email = email
        self.tx_ref = uuidGenerator()
        self.fullName = fullName
        self.network = network
        self.redirect_url = redirect_url
        self.description = description

    def chargeBodyObject(self) -> dict:
        return {
            "amount": self.amount,
            "currency": self.currency,
            "phone_number": self.phone_number,
            "email": self.email,
            "tx_ref": self.tx_ref,
            "fullName": self.fullName,
            "network": self.network,
            "redirect_url": self.redirect_url,
            "description": self.description
        }






class CustomeException(Exception):
    """
    This class is the base of custom exception which inherits Exception class
    """
    pass


class RequestError(CustomeException):
    """
    This is a custom exception error which you can raise when there is an error of connetion
    while using the requests library
    """
    pass
